import { Citation } from "./citation";
import { Member } from "./member";
import { Vertex } from "../graph.interface";
import { randomUUID } from "crypto";

export interface AuthorWeight {
    weight: number;
    author: Member
}

export interface CitationWeight {
    weight: number;
    citation: Citation;
}

export class Post implements Vertex {

    private static DEFAULT_WEIGHT = 1000;
    private static localStore = new Map<string, Post>;

    public static getAllPosts() {
        return Array.from(Post.localStore.values());
    }

    public static getPost(id: string) {
        return Post.localStore.get(id);
    }

    public static serialize(post: Post, pretty: boolean = true) {
        const postProperties = {
            id: post.id,
            title: post.title,
            content: post.content,
            authors: post.authors.map(author => { return {
                author: author.id,
                weight: post.getWeight(author)
            }}),
            citations: post.citations.map(citation => { return {
                citation: citation.id,
                weight: post.getWeight(citation)
            }})
        }
        if (!pretty) return JSON.stringify(postProperties);
        return JSON.stringify(postProperties, undefined, 2);
    }

    public static parse(serializedPost: string) {
        try {
            const properties = JSON.parse(serializedPost);
            const id = properties.id;
            const title = properties.title;
            const content = properties.content;
            const authors = properties.authors.map(({author, weight}) => {
                author = Member.getMember(author) || new Member(author);
                return {author, weight};
            });
            const citations = properties.citations.map(({citation, weight}) => {
                citation = Citation.getCitation(citation);
                if (citation) return {citation, weight};
            });
            const post = new Post(id, title, content, authors[0].author);
            for (const author of authors) post.setAuthor(author.author, author.weight);
            for (const citation of citations) post.setCitation(citation.citation, citation.weight);
            return post;
        } catch(err) {
            throw new Error("Failed to parse the serialized post. \n  - " + err.message);
        }
    }

    private _id: string;
    private _title: string;
    private _content: string;
    private _authors: AuthorWeight[];
    private _citations: CitationWeight[];
    private _authorsWeightTotal: number;
    private _citationsWeightTotal: number;

    constructor (id: string | null, title: string, content: string, authors: Member | Member[]) {
        this._id = id ? id : randomUUID();
        this._title = title;
        this._content = content;
        this._authors = [];
        this._citations = [];
        this._authorsWeightTotal = 0;
        this._citationsWeightTotal = 0;
        if (!Array.isArray(authors)) authors = [authors];
        for (const author of <Member[]> authors) {
            this._authorsWeightTotal += Post.DEFAULT_WEIGHT;
            this._authors.push({
                weight: Post.DEFAULT_WEIGHT,
                author: author
            });
        }
        Post.localStore.set(this._id, this);
    }

    public get id() { return this._id; }
    public get edges() { return this.citations; }
    public get data() { return Post.serialize(this); }

    public get title() { return this._title; }
    public get content() { return this._content; }
    public get authors() { return this._authors.map(({author}) => author); }
    public get citations() { return this._citations.map(({citation}) => citation); }
    public get authorsWeightTotal() { return this._authorsWeightTotal; }
    public get citationsWeightTotal() { return this._citationsWeightTotal; }

    public setAuthor(author: Member, weight: number = Post.DEFAULT_WEIGHT) {
        const index = this.authors.indexOf(author);
        if (index != -1) {
            const current = this._authors[index];
            this._authorsWeightTotal -= current.weight;
            this._authors.splice(index, 1);
        }
        this._authors.push({author, weight});
        this._authorsWeightTotal += weight;
    }

    public removeAuthor(author: Member) {
        if (this.authors.length == 1 ) throw new Error("Cannot remove the last author of a post.");
        const index = this.authors.indexOf(author);
        if (index != -1) {
            const current = this._authors[index];
            this._authorsWeightTotal -= current.weight;
            this._authors.splice(index, 1);
        }
    }

    public setCitation(citation: Citation, weight: number = Post.DEFAULT_WEIGHT) {
        this.removeCitation(citation);
        this._citations.push({citation, weight});
        this._citationsWeightTotal += weight;
    }


    public removeCitation(citation: Citation) {
        const index = this.citations.indexOf(citation);
        if (index != -1) {
            const current = this._citations[index];
            this._citationsWeightTotal -= current.weight;
            this._citations.splice(index, 1);
        }
    }

    public getWeight(weighted: Member | Citation) {
        const forAuthor = weighted instanceof Member;
        const arr = forAuthor ? this._authors  : this._citations;
        const key = forAuthor ? "author" : "citation";
        const index = arr.findIndex(item => item[key] == weighted);
        if (index == -1) return -1;
        return arr[index].weight;
    }

}